/*
  This sketch uses the libraries "Dabble" and "U8g2".
  Check if these are present in Documents\Arduino\libraries.
  If not, install them via Tools > Manage Libraries
  
  Uses Dabble in Gamepad mode (libraries\Dabble\src\GamePadModule.h)

  Info about Gamepad functions: https://thestempedia.com/docs/dabble/game-pad-module/
  This version adds support for accelerometer version of the gamepad.

  This sketch receives commands via Software serial (Dabble) and show them on a display

  Connect USB cable then Upload this sketch.
  Open the Serial Monitor to view test output. Make sure the speed in the Serial Monitor is set to 9600.
 */

// dabble:
#define CUSTOM_SETTINGS
#define INCLUDE_GAMEPAD_MODULE
#include <Dabble.h>

// oled display:
#include <U8g2lib.h>

U8X8_SSD1306_128X64_NONAME_HW_I2C display(U8X8_PIN_NONE);

bool connected=false;

// variables used as text buffers for input received:
String line;
char buf[16];


void setup() {
  // Open serial communications
  Serial.begin(9600);

  // Have Dabble connect to Bluetooth Module:
  Dabble.begin(9600, 0, 1); // Baudrate, RX, TX to which bluetooth module is connected, 0,1 = default RX,TX of Uno, Nano
  
  display.begin();
  display.setPowerSave(0);

  display.setFont(u8x8_font_pxplusibmcgathin_f);
  display.drawString(0,0,"Connect");
  display.drawString(0,1,"bluetooth...");
}

void loop() {
  if (Dabble.isAppConnected() && !connected) {
    display.clearDisplay();
    display.drawString(0,0,"Connected");
    connected = true;
  }
  Dabble.processInput();
  if (GamePad.isStartPressed()) message("start");
  else if (GamePad.isSelectPressed()) message("select");
  else if (GamePad.isUpPressed()) message("up");
  else if (GamePad.isDownPressed()) message("down");
  else if (GamePad.isLeftPressed()) message("left");
  else if (GamePad.isRightPressed()) message("right");
  else if (GamePad.isSquarePressed()) message("stop");
  else if (GamePad.isTrianglePressed()) message("triangle");
  else if (GamePad.isCrossPressed()) message("cross");
  else if (GamePad.isCirclePressed()) message("circle");

  // read accelerometer data from Dabble:
  int a = GamePad.getAngle();
  int b = GamePad.getRadius();
  float c = GamePad.getXaxisData();
  float d = GamePad.getYaxisData();

  // print data to the display:
  display.clearLine(2);
  display.drawString(0,2,"Angle: ");
  display.setCursor(7, 2);
  display.print(a);

  display.clearLine(3);
  display.drawString(0,3,"Radius: ");
  display.setCursor(7, 3);
  display.print(b);

  display.clearLine(4);
  display.drawString(0,4,"x_axis: ");
  display.setCursor(7, 4);
  display.print(c);

  display.clearLine(5);
  display.drawString(0,5,"y_axis: ");
  display.setCursor(7, 5);
  display.print(d);

  if (d>=1) {
    message("forward");
  }
  else if (d<=-1) {
    message("backward");
  }
  else {
    if(a>0 && a<270) message("left");
    else if(a>270) message("right");
  }

  delay(50);

  // receive data from Serial connection:
  while (Serial.available()>0) {
       char readchar = Serial.read();
       if (readchar>=32 && readchar<127) line += readchar; // if valid character add it to line (ignore/drop invalids chars)
       delay(2);
  }
  if (line.length() > 1) { // did we receive anything?
    // Print text we received on the display:

    line = ""; // make line empty to prevent processing the same line twice
  }
  delay(50);
}

/*
 * Show a message on line 1 of the display and also show it on Serial output.
 */
void message(char* message) {
  display.clearLine(1); // clear line 1 of the display
  display.drawString(0,1,message); // print message on line 1 of the display
  Serial.println(message);  // print message to serial connection
}

/*
 * Get first part of String s
 */
String getPart1(String s) {
  int space = s.lastIndexOf(' '); // is there a space? (' ')
  if (space>0) return s.substring(0, space); // return part until the space
  return s; // if there was no space, just return the whole String
}

/*
 * Get second part of String s
 */
String getPart2(String s) {
  int space = s.lastIndexOf(' '); // is there a space? (' ')
  if (space>0) return s.substring(space+1); // return part after the space
  return ""; // if there was no space, return empty String
}
